Package: wrGraph
Version: 1.3.16
Title: Graphics in the Context of Analyzing High-Throughput Data
Authors@R: person("Wolfgang", "Raffelsberger", email = "w.raffelsberger@gmail.com", role = c("aut", "cre"))
Author: Wolfgang Raffelsberger [aut, cre]
Maintainer: Wolfgang Raffelsberger <w.raffelsberger@gmail.com>
Description: Additional options for making graphics in the context of analyzing high-throughput data are available here.
  This includes automatic segmenting of the current device (eg window) to accommodate multiple new plots, 
  automatic checking for optimal location of legends in plots, small histograms to insert as legends, 
  histograms re-transforming axis labels to linear when plotting log2-transformed data,
  a violin-plot <doi:10.1080/00031305.1998.10480559> function for a wide variety of input-formats, 
  principal components analysis (PCA) <doi:10.1080/14786440109462720> with bag-plots <doi:10.1080/00031305.1999.10474494> to highlight and compare the center areas for groups of samples, 
  generic MA-plots (differential- versus average-value plots) <doi:10.1093/nar/30.4.e15>, 
  staggered count plots and generation of mouse-over interactive html pages.
Depends: R (>= 3.5.0)
Imports: graphics, grDevices, grid, lattice, RColorBrewer, stats,
        wrMisc (>= 2.0.1)
Suggests: dplyr, factoextra, FactoMineR, ggpubr, knitr, limma,
        rmarkdown, sm
License: GPL-3
Encoding: UTF-8
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-11 10:59:12 UTC; wraff
Repository: CRAN
Date/Publication: 2026-06-11 12:10:02 UTC
Built: R 4.4.3; ; 2026-06-11 15:54:48 UTC; unix
