# Generated by roxygen2: do not edit by hand

export(.addTextToPoints)
export(.bestLegendLoc)
export(.determFigMargPix)
export(.predPointsPix)
export(.serachColName)
export(MAplotW)
export(VolcanoPlotW)
export(addBagPlot)
export(checkForLegLoc)
export(convertPlotCoordPix)
export(cumFrqPlot)
export(foldChangeArrow)
export(histW)
export(imageW)
export(legendHist)
export(mouseOverHtmlFile)
export(partitionPlot)
export(plotBy2Groups)
export(plotLinReg)
export(plotPCAw)
export(plotW2Leg)
export(profileAsClu)
export(staggerdCountsPlot)
export(vioplotW)
